@@ -1,20 +1,32 @@
 /* libFLAC++ - Free Lossless Audio Codec library
- * Copyright (C) 2002,2003  Josh Coalson
+ * Copyright (C) 2002,2003,2004  Josh Coalson
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * - Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * - Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ *
+ * - Neither the name of the Xiph.org Foundation nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "FLAC++/metadata.h"
@@ -22,6 +34,11 @@
 #include <stdlib.h> // for malloc(), free()
 #include <string.h> // for memcpy() etc.
 
+#ifdef _MSC_VER
+// warning C4800: 'int' : forcing to bool 'true' or 'false' (performance warning)
+#pragma warning ( disable : 4800 )
+#endif
+
 namespace FLAC {
 	namespace Metadata {
 
@@ -968,10 +985,26 @@ namespace FLAC {
 		{
 			FLAC__ASSERT(0 != filename);
 
-			::FLAC__StreamMetadata s;
+			::FLAC__StreamMetadata object;
 
-			if(::FLAC__metadata_get_streaminfo(filename, &s)) {
-				streaminfo = s;
+			if(::FLAC__metadata_get_streaminfo(filename, &object)) {
+				streaminfo = object;
+				return true;
+			}
+			else
+				return false;
+		}
+
+		FLACPP_API bool get_tags(const char *filename, VorbisComment *&tags)
+		{
+			FLAC__ASSERT(0 != filename);
+
+			::FLAC__StreamMetadata *object;
+
+			tags = 0;
+
+			if(::FLAC__metadata_get_tags(filename, &object)) {
+				tags = new VorbisComment(object, /*copy=*/false);
 				return true;
 			}
 			else
@@ -1110,12 +1143,36 @@ namespace FLAC {
 			return (bool)::FLAC__metadata_chain_read(chain_, filename);
 		}
 
+		bool Chain::read(FLAC__IOHandle handle, ::FLAC__IOCallbacks callbacks)
+		{
+			FLAC__ASSERT(is_valid());
+			return (bool)::FLAC__metadata_chain_read_with_callbacks(chain_, handle, callbacks);
+		}
+
+		bool Chain::check_if_tempfile_needed(bool use_padding)
+		{
+			FLAC__ASSERT(is_valid());
+			return (bool)::FLAC__metadata_chain_check_if_tempfile_needed(chain_, use_padding);
+		}
+
 		bool Chain::write(bool use_padding, bool preserve_file_stats)
 		{
 			FLAC__ASSERT(is_valid());
 			return (bool)::FLAC__metadata_chain_write(chain_, use_padding, preserve_file_stats);
 		}
 
+		bool Chain::write(bool use_padding, ::FLAC__IOHandle handle, ::FLAC__IOCallbacks callbacks)
+		{
+			FLAC__ASSERT(is_valid());
+			return (bool)::FLAC__metadata_chain_write_with_callbacks(chain_, use_padding, handle, callbacks);
+		}
+
+		bool Chain::write(bool use_padding, ::FLAC__IOHandle handle, ::FLAC__IOCallbacks callbacks, ::FLAC__IOHandle temp_handle, ::FLAC__IOCallbacks temp_callbacks)
+		{
+			FLAC__ASSERT(is_valid());
+			return (bool)::FLAC__metadata_chain_write_with_callbacks_and_tempfile(chain_, use_padding, handle, callbacks, temp_handle, temp_callbacks);
+		}
+
 		void Chain::merge_padding()
 		{
 			FLAC__ASSERT(is_valid());
