@@ -1,33 +1,38 @@
 /* libOggFLAC - Free Lossless Audio Codec + Ogg library
- * Copyright (C) 2002,2003  Josh Coalson
+ * Copyright (C) 2002,2003,2004  Josh Coalson
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * - Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * - Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ *
+ * - Neither the name of the Xiph.org Foundation nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdlib.h> /* for calloc() */
-#include <string.h> /* for memset() */
-#include <ogg/ogg.h>
 #include "FLAC/assert.h"
 #include "protected/stream_decoder.h"
 
-#ifdef min
-#undef min
-#endif
-#define min(x,y) ((x)<(y)?(x):(y))
-
 /***********************************************************************
  *
  * Private class method prototypes
@@ -39,6 +44,7 @@ static FLAC__StreamDecoderReadStatus read_callback_(const FLAC__StreamDecoder *d
 static FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__StreamDecoder *decoder, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data);
 static void metadata_callback_(const FLAC__StreamDecoder *decoder, const FLAC__StreamMetadata *metadata, void *client_data);
 static void error_callback_(const FLAC__StreamDecoder *decoder, FLAC__StreamDecoderErrorStatus status, void *client_data);
+static OggFLAC__OggDecoderAspectReadStatus read_callback_proxy_(const void *void_decoder, FLAC__byte buffer[], unsigned *bytes, void *client_data);
 
 
 /***********************************************************************
@@ -54,11 +60,6 @@ typedef struct OggFLAC__StreamDecoderPrivate {
 	OggFLAC__StreamDecoderErrorCallback error_callback;
 	void *client_data;
 	FLAC__StreamDecoder *FLAC_stream_decoder;
-	struct {
-		ogg_stream_state stream_state;
-		ogg_sync_state sync_state;
-		FLAC__bool need_serial_number;
-	} ogg;
 } OggFLAC__StreamDecoderPrivate;
 
 /***********************************************************************
@@ -67,14 +68,15 @@ typedef struct OggFLAC__StreamDecoderPrivate {
  *
  ***********************************************************************/
 
-const OggFLAC_API char * const OggFLAC__StreamDecoderStateString[] = {
+OggFLAC_API const char * const OggFLAC__StreamDecoderStateString[] = {
 	"OggFLAC__STREAM_DECODER_OK",
+	"OggFLAC__STREAM_DECODER_END_OF_STREAM",
 	"OggFLAC__STREAM_DECODER_OGG_ERROR",
 	"OggFLAC__STREAM_DECODER_READ_ERROR",
 	"OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR",
-	"OggFLAC__STREAM_DECODER_INVALID_CALLBACK",
 	"OggFLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR",
 	"OggFLAC__STREAM_DECODER_ALREADY_INITIALIZED",
+	"OggFLAC__STREAM_DECODER_INVALID_CALLBACK",
 	"OggFLAC__STREAM_DECODER_UNINITIALIZED"
 };
 
@@ -153,12 +155,7 @@ OggFLAC_API OggFLAC__StreamDecoderState OggFLAC__stream_decoder_init(OggFLAC__St
 	if(0 == decoder->private_->read_callback || 0 == decoder->private_->write_callback || 0 == decoder->private_->metadata_callback || 0 == decoder->private_->error_callback)
 		return decoder->protected_->state = OggFLAC__STREAM_DECODER_INVALID_CALLBACK;
 
-	decoder->private_->ogg.need_serial_number = decoder->protected_->use_first_serial_number;
-	/* we will determine the serial number later if necessary */
-	if(ogg_stream_init(&decoder->private_->ogg.stream_state, decoder->protected_->serial_number) != 0)
-		return decoder->protected_->state = OggFLAC__STREAM_DECODER_OGG_ERROR;
-
-	if(ogg_sync_init(&decoder->private_->ogg.sync_state) != 0)
+	if(!OggFLAC__ogg_decoder_aspect_init(&decoder->protected_->ogg_decoder_aspect))
 		return decoder->protected_->state = OggFLAC__STREAM_DECODER_OGG_ERROR;
 
 	FLAC__stream_decoder_set_read_callback(decoder->private_->FLAC_stream_decoder, read_callback_);
@@ -186,8 +183,7 @@ OggFLAC_API void OggFLAC__stream_decoder_finish(OggFLAC__StreamDecoder *decoder)
 
 	FLAC__stream_decoder_finish(decoder->private_->FLAC_stream_decoder);
 
-	(void)ogg_sync_clear(&decoder->private_->ogg.sync_state);
-	(void)ogg_stream_clear(&decoder->private_->ogg.stream_state);
+	OggFLAC__ogg_decoder_aspect_finish(&decoder->protected_->ogg_decoder_aspect);
 
 	set_defaults_(decoder);
 
@@ -256,8 +252,7 @@ OggFLAC_API FLAC__bool OggFLAC__stream_decoder_set_serial_number(OggFLAC__Stream
 	FLAC__ASSERT(0 != decoder->protected_);
 	if(decoder->protected_->state != OggFLAC__STREAM_DECODER_UNINITIALIZED)
 		return false;
-	decoder->protected_->use_first_serial_number = false;
-	decoder->protected_->serial_number = value;
+	OggFLAC__ogg_decoder_aspect_set_serial_number(&decoder->protected_->ogg_decoder_aspect, value);
 	return true;
 }
 
@@ -335,6 +330,14 @@ OggFLAC_API FLAC__StreamDecoderState OggFLAC__stream_decoder_get_FLAC_stream_dec
 	return FLAC__stream_decoder_get_state(decoder->private_->FLAC_stream_decoder);
 }
 
+OggFLAC_API const char *OggFLAC__stream_decoder_get_resolved_state_string(const OggFLAC__StreamDecoder *decoder)
+{
+	if(decoder->protected_->state != OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR)
+		return OggFLAC__StreamDecoderStateString[decoder->protected_->state];
+	else
+		return FLAC__stream_decoder_get_resolved_state_string(decoder->private_->FLAC_stream_decoder);
+}
+
 OggFLAC_API unsigned OggFLAC__stream_decoder_get_channels(const OggFLAC__StreamDecoder *decoder)
 {
 	FLAC__ASSERT(0 != decoder);
@@ -376,7 +379,7 @@ OggFLAC_API FLAC__bool OggFLAC__stream_decoder_flush(OggFLAC__StreamDecoder *dec
 	FLAC__ASSERT(0 != decoder->private_);
 	FLAC__ASSERT(0 != decoder->protected_);
 
-	(void)ogg_sync_clear(&decoder->private_->ogg.sync_state);
+	OggFLAC__ogg_decoder_aspect_flush(&decoder->protected_->ogg_decoder_aspect);
 
 	if(!FLAC__stream_decoder_flush(decoder->private_->FLAC_stream_decoder)) {
 		decoder->protected_->state = OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR;
@@ -399,6 +402,8 @@ OggFLAC_API FLAC__bool OggFLAC__stream_decoder_reset(OggFLAC__StreamDecoder *dec
 		return false;
 	}
 
+	OggFLAC__ogg_decoder_aspect_reset(&decoder->protected_->ogg_decoder_aspect);
+
 	if(!FLAC__stream_decoder_reset(decoder->private_->FLAC_stream_decoder)) {
 		decoder->protected_->state = OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR;
 		return false;
@@ -411,23 +416,62 @@ OggFLAC_API FLAC__bool OggFLAC__stream_decoder_reset(OggFLAC__StreamDecoder *dec
 
 OggFLAC_API FLAC__bool OggFLAC__stream_decoder_process_single(OggFLAC__StreamDecoder *decoder)
 {
+	FLAC__bool ret;
 	FLAC__ASSERT(0 != decoder);
-	FLAC__ASSERT(0 != decoder->private_);
-	return FLAC__stream_decoder_process_single(decoder->private_->FLAC_stream_decoder);
+
+	if(FLAC__stream_decoder_get_state(decoder->private_->FLAC_stream_decoder) == FLAC__STREAM_DECODER_END_OF_STREAM)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_END_OF_STREAM;
+
+	if(decoder->protected_->state == OggFLAC__STREAM_DECODER_END_OF_STREAM)
+		return true;
+
+	FLAC__ASSERT(decoder->protected_->state == OggFLAC__STREAM_DECODER_OK);
+
+	ret = FLAC__stream_decoder_process_single(decoder->private_->FLAC_stream_decoder);
+	if(!ret)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR;
+
+	return ret;
 }
 
 OggFLAC_API FLAC__bool OggFLAC__stream_decoder_process_until_end_of_metadata(OggFLAC__StreamDecoder *decoder)
 {
+	FLAC__bool ret;
 	FLAC__ASSERT(0 != decoder);
-	FLAC__ASSERT(0 != decoder->private_);
-	return FLAC__stream_decoder_process_until_end_of_metadata(decoder->private_->FLAC_stream_decoder);
+
+	if(FLAC__stream_decoder_get_state(decoder->private_->FLAC_stream_decoder) == FLAC__STREAM_DECODER_END_OF_STREAM)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_END_OF_STREAM;
+
+	if(decoder->protected_->state == OggFLAC__STREAM_DECODER_END_OF_STREAM)
+		return true;
+
+	FLAC__ASSERT(decoder->protected_->state == OggFLAC__STREAM_DECODER_OK);
+
+	ret = FLAC__stream_decoder_process_until_end_of_metadata(decoder->private_->FLAC_stream_decoder);
+	if(!ret)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR;
+
+	return ret;
 }
 
 OggFLAC_API FLAC__bool OggFLAC__stream_decoder_process_until_end_of_stream(OggFLAC__StreamDecoder *decoder)
 {
+	FLAC__bool ret;
 	FLAC__ASSERT(0 != decoder);
-	FLAC__ASSERT(0 != decoder->private_);
-	return FLAC__stream_decoder_process_until_end_of_stream(decoder->private_->FLAC_stream_decoder);
+
+	if(FLAC__stream_decoder_get_state(decoder->private_->FLAC_stream_decoder) == FLAC__STREAM_DECODER_END_OF_STREAM)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_END_OF_STREAM;
+
+	if(decoder->protected_->state == OggFLAC__STREAM_DECODER_END_OF_STREAM)
+		return true;
+
+	FLAC__ASSERT(decoder->protected_->state == OggFLAC__STREAM_DECODER_OK);
+
+	ret = FLAC__stream_decoder_process_until_end_of_stream(decoder->private_->FLAC_stream_decoder);
+	if(!ret)
+		decoder->protected_->state = OggFLAC__STREAM_DECODER_FLAC_STREAM_DECODER_ERROR;
+
+	return ret;
 }
 
 
@@ -444,61 +488,40 @@ void set_defaults_(OggFLAC__StreamDecoder *decoder)
 	decoder->private_->metadata_callback = 0;
 	decoder->private_->error_callback = 0;
 	decoder->private_->client_data = 0;
-	decoder->protected_->use_first_serial_number = true;
+	OggFLAC__ogg_decoder_aspect_set_defaults(&decoder->protected_->ogg_decoder_aspect);
 }
 
 FLAC__StreamDecoderReadStatus read_callback_(const FLAC__StreamDecoder *unused, FLAC__byte buffer[], unsigned *bytes, void *client_data)
 {
-	static const unsigned OGG_BYTES_CHUNK = 8192;
 	OggFLAC__StreamDecoder *decoder = (OggFLAC__StreamDecoder*)client_data;
-	unsigned ogg_bytes_to_read, ogg_bytes_read;
-	ogg_page page;
-	char *oggbuf;
 
 	(void)unused;
 
-	/*
-	 * We have to be careful not to read in more than the
-	 * FLAC__StreamDecoder says it has room for.  We know
-	 * that the size of the decoded data must be no more
-	 * than the encoded data we will read.
-	 */
-	ogg_bytes_to_read = min(*bytes, OGG_BYTES_CHUNK);
-	oggbuf = ogg_sync_buffer(&decoder->private_->ogg.sync_state, ogg_bytes_to_read);
-
-	if(decoder->private_->read_callback(decoder, oggbuf, &ogg_bytes_to_read, decoder->private_->client_data) != FLAC__STREAM_DECODER_READ_STATUS_CONTINUE) {
-		decoder->protected_->state = OggFLAC__STREAM_DECODER_READ_ERROR;
-		return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
-	}
-	ogg_bytes_read = ogg_bytes_to_read;
-
-	if(ogg_sync_wrote(&decoder->private_->ogg.sync_state, ogg_bytes_read) < 0) {
-		decoder->protected_->state = OggFLAC__STREAM_DECODER_READ_ERROR;
-		return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
-	}
-
-	*bytes = 0;
-	while(ogg_sync_pageout(&decoder->private_->ogg.sync_state, &page) == 1) {
-		/* grab the serial number if necessary */
-		if(decoder->private_->ogg.need_serial_number) {
-			decoder->private_->ogg.stream_state.serialno = decoder->protected_->serial_number = ogg_page_serialno(&page);
-			decoder->private_->ogg.need_serial_number = false;
-		}
-		if(ogg_stream_pagein(&decoder->private_->ogg.stream_state, &page) == 0) {
-			ogg_packet packet;
-
-			while(ogg_stream_packetout(&decoder->private_->ogg.stream_state, &packet) == 1) {
-				memcpy(buffer, packet.packet, packet.bytes);
-				*bytes += packet.bytes;
-				buffer += packet.bytes;
-			}
-		} else {
+	switch(OggFLAC__ogg_decoder_aspect_read_callback_wrapper(&decoder->protected_->ogg_decoder_aspect, buffer, bytes, read_callback_proxy_, decoder, decoder->private_->client_data)) {
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_OK:
+			return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+		/* we don't really have a way to handle lost sync via read
+		 * callback so we'll let it pass and let the underlying
+		 * FLAC decoder catch the error
+		 */
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_LOST_SYNC:
+			return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_END_OF_STREAM:
+			return FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM;
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_NOT_FLAC:
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_UNSUPPORTED_MAPPING_VERSION:
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_ABORT:
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_ERROR:
 			decoder->protected_->state = OggFLAC__STREAM_DECODER_READ_ERROR;
 			return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
-		}
+		case OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_MEMORY_ALLOCATION_ERROR:
+			decoder->protected_->state = OggFLAC__STREAM_DECODER_MEMORY_ALLOCATION_ERROR;
+			return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
+		default:
+			FLAC__ASSERT(0);
+			/* double protection */
+			return FLAC__STREAM_DECODER_READ_STATUS_ABORT;
 	}
-
-	return FLAC__STREAM_DECODER_READ_STATUS_CONTINUE;
 }
 
 FLAC__StreamDecoderWriteStatus write_callback_(const FLAC__StreamDecoder *unused, const FLAC__Frame *frame, const FLAC__int32 * const buffer[], void *client_data)
@@ -521,3 +544,21 @@ void error_callback_(const FLAC__StreamDecoder *unused, FLAC__StreamDecoderError
 	(void)unused;
 	decoder->private_->error_callback(decoder, status, decoder->private_->client_data);
 }
+
+OggFLAC__OggDecoderAspectReadStatus read_callback_proxy_(const void *void_decoder, FLAC__byte buffer[], unsigned *bytes, void *client_data)
+{
+	OggFLAC__StreamDecoder *decoder = (OggFLAC__StreamDecoder*)void_decoder;
+
+	switch(decoder->private_->read_callback(decoder, buffer, bytes, client_data)) {
+		case FLAC__STREAM_DECODER_READ_STATUS_CONTINUE:
+			return OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_OK;
+		case FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM:
+			return OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_END_OF_STREAM;
+		case FLAC__STREAM_DECODER_READ_STATUS_ABORT:
+			return OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_ABORT;
+		default:
+			/* double protection: */
+			FLAC__ASSERT(0);
+			return OggFLAC__OGG_DECODER_ASPECT_READ_STATUS_ABORT;
+	}
+}
