@@ -1,28 +1,38 @@
 /* libOggFLAC - Free Lossless Audio Codec + Ogg library
- * Copyright (C) 2002,2003  Josh Coalson
+ * Copyright (C) 2002,2003,2004  Josh Coalson
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * - Redistributions of source code must retain the above copyright
+ * notice, this list of conditions and the following disclaimer.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * - Redistributions in binary form must reproduce the above copyright
+ * notice, this list of conditions and the following disclaimer in the
+ * documentation and/or other materials provided with the distribution.
+ *
+ * - Neither the name of the Xiph.org Foundation nor the names of its
+ * contributors may be used to endorse or promote products derived from
+ * this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include <stdio.h>
 #include <stdlib.h> /* for calloc() */
-#include <string.h> /* for memcpy() */
-#include <ogg/ogg.h>
 #include "FLAC/assert.h"
-#include "OggFLAC/stream_decoder.h"
+#include "OggFLAC/stream_encoder.h"
 #include "protected/stream_encoder.h"
 
 /***********************************************************************
@@ -49,15 +59,9 @@ static void metadata_callback_(const FLAC__StreamEncoder *encoder, const FLAC__S
 
 typedef struct OggFLAC__StreamEncoderPrivate {
 	OggFLAC__StreamEncoderWriteCallback write_callback;
+	OggFLAC__StreamEncoderMetadataCallback metadata_callback;
 	void *client_data;
 	FLAC__StreamEncoder *FLAC_stream_encoder;
-	/* internal vars (all the above are class settings) */
-	FLAC__bool is_first_packet;
-	FLAC__uint64 samples_written;
-	struct {
-		ogg_stream_state stream_state;
-		ogg_page page;
-	} ogg;
 } OggFLAC__StreamEncoderPrivate;
 
 
@@ -67,7 +71,7 @@ typedef struct OggFLAC__StreamEncoderPrivate {
  *
  ***********************************************************************/
 
-const OggFLAC_API char * const OggFLAC__StreamEncoderStateString[] = {
+OggFLAC_API const char * const OggFLAC__StreamEncoderStateString[] = {
 	"OggFLAC__STREAM_ENCODER_OK",
 	"OggFLAC__STREAM_ENCODER_OGG_ERROR",
 	"OggFLAC__STREAM_ENCODER_FLAC_STREAM_ENCODER_ERROR",
@@ -150,11 +154,11 @@ OggFLAC_API OggFLAC__StreamEncoderState OggFLAC__stream_encoder_init(OggFLAC__St
 	if(encoder->protected_->state != OggFLAC__STREAM_ENCODER_UNINITIALIZED)
 		return encoder->protected_->state = OggFLAC__STREAM_ENCODER_ALREADY_INITIALIZED;
 
-	if(0 == encoder->private_->write_callback)
+	if(0 == encoder->private_->write_callback || 0 == encoder->private_->metadata_callback)
 		return encoder->protected_->state = OggFLAC__STREAM_ENCODER_INVALID_CALLBACK;
 
-	if(ogg_stream_init(&encoder->private_->ogg.stream_state, encoder->protected_->serial_number) != 0)
-		return encoder->protected_->state = OggFLAC__STREAM_ENCODER_OGG_ERROR;
+	if(!OggFLAC__ogg_encoder_aspect_init(&encoder->protected_->ogg_encoder_aspect))
+			return encoder->protected_->state = OggFLAC__STREAM_ENCODER_OGG_ERROR;
 
 	FLAC__stream_encoder_set_write_callback(encoder->private_->FLAC_stream_encoder, write_callback_);
 	FLAC__stream_encoder_set_metadata_callback(encoder->private_->FLAC_stream_encoder, metadata_callback_);
@@ -163,9 +167,6 @@ OggFLAC_API OggFLAC__StreamEncoderState OggFLAC__stream_encoder_init(OggFLAC__St
 	if(FLAC__stream_encoder_init(encoder->private_->FLAC_stream_encoder) != FLAC__STREAM_ENCODER_OK)
 		return encoder->protected_->state = OggFLAC__STREAM_ENCODER_FLAC_STREAM_ENCODER_ERROR;
 
-	encoder->private_->is_first_packet = true;
-	encoder->private_->samples_written = 0;
-
 	return encoder->protected_->state = OggFLAC__STREAM_ENCODER_OK;
 }
 
@@ -182,7 +183,7 @@ OggFLAC_API void OggFLAC__stream_encoder_finish(OggFLAC__StreamEncoder *encoder)
 
 	FLAC__stream_encoder_finish(encoder->private_->FLAC_stream_encoder);
 
-	(void)ogg_stream_clear(&encoder->private_->ogg.stream_state);
+	OggFLAC__ogg_encoder_aspect_finish(&encoder->protected_->ogg_encoder_aspect);
 
 	set_defaults_(encoder);
 
@@ -197,7 +198,7 @@ OggFLAC_API FLAC__bool OggFLAC__stream_encoder_set_serial_number(OggFLAC__Stream
 	FLAC__ASSERT(0 != encoder->private_->FLAC_stream_encoder);
 	if(encoder->protected_->state != OggFLAC__STREAM_ENCODER_UNINITIALIZED)
 		return false;
-	encoder->protected_->serial_number = value;
+	OggFLAC__ogg_encoder_aspect_set_serial_number(&encoder->protected_->ogg_encoder_aspect, value);
 	return true;
 }
 
@@ -396,6 +397,21 @@ OggFLAC_API FLAC__bool OggFLAC__stream_encoder_set_metadata(OggFLAC__StreamEncod
 	FLAC__ASSERT(0 != encoder->private_->FLAC_stream_encoder);
 	if(encoder->protected_->state != OggFLAC__STREAM_ENCODER_UNINITIALIZED)
 		return false;
+	/* reorder metadata if necessary to ensure that any VORBIS_COMMENT is the first, according to the mapping spec */
+	if(0 != metadata && num_blocks > 1) {
+		unsigned i;
+		for(i = 1; i < num_blocks; i++) {
+			if(0 != metadata[i] && metadata[i]->type == FLAC__METADATA_TYPE_VORBIS_COMMENT) {
+				FLAC__StreamMetadata *vc = metadata[i];
+				for( ; i > 0; i--)
+					metadata[i] = metadata[i-1];
+				metadata[0] = vc;
+				break;
+			}
+		}
+	}
+	if(!OggFLAC__ogg_encoder_aspect_set_num_metadata(&encoder->protected_->ogg_encoder_aspect, num_blocks))
+		return false;
 	return FLAC__stream_encoder_set_metadata(encoder->private_->FLAC_stream_encoder, metadata, num_blocks);
 }
 
@@ -411,6 +427,18 @@ OggFLAC_API FLAC__bool OggFLAC__stream_encoder_set_write_callback(OggFLAC__Strea
 	return true;
 }
 
+OggFLAC_API FLAC__bool OggFLAC__stream_encoder_set_metadata_callback(OggFLAC__StreamEncoder *encoder, OggFLAC__StreamEncoderMetadataCallback value)
+{
+	FLAC__ASSERT(0 != encoder);
+	FLAC__ASSERT(0 != encoder->private_);
+	FLAC__ASSERT(0 != encoder->protected_);
+	FLAC__ASSERT(0 != value);
+	if(encoder->protected_->state != OggFLAC__STREAM_ENCODER_UNINITIALIZED)
+		return false;
+	encoder->private_->metadata_callback = value;
+	return true;
+}
+
 OggFLAC_API FLAC__bool OggFLAC__stream_encoder_set_client_data(OggFLAC__StreamEncoder *encoder, void *value)
 {
 	FLAC__ASSERT(0 != encoder);
@@ -480,6 +508,14 @@ OggFLAC_API FLAC__StreamDecoderState OggFLAC__stream_encoder_get_verify_decoder_
 	return FLAC__stream_encoder_get_verify_decoder_state(encoder->private_->FLAC_stream_encoder);
 }
 
+OggFLAC_API const char *OggFLAC__stream_encoder_get_resolved_state_string(const OggFLAC__StreamEncoder *encoder)
+{
+	if(encoder->protected_->state != OggFLAC__STREAM_ENCODER_FLAC_STREAM_ENCODER_ERROR)
+		return OggFLAC__StreamEncoderStateString[encoder->protected_->state];
+	else
+		return FLAC__stream_encoder_get_resolved_state_string(encoder->private_->FLAC_stream_encoder);
+}
+
 OggFLAC_API void OggFLAC__stream_encoder_get_verify_decoder_error_stats(const OggFLAC__StreamEncoder *encoder, FLAC__uint64 *absolute_sample, unsigned *frame_number, unsigned *channel, unsigned *sample, FLAC__int32 *expected, FLAC__int32 *got)
 {
 	FLAC__ASSERT(0 != encoder);
@@ -650,60 +686,28 @@ void set_defaults_(OggFLAC__StreamEncoder *encoder)
 	FLAC__ASSERT(0 != encoder);
 
 	encoder->private_->write_callback = 0;
+	encoder->private_->metadata_callback = 0;
 	encoder->private_->client_data = 0;
-	encoder->protected_->serial_number = 0;
+	OggFLAC__ogg_encoder_aspect_set_defaults(&encoder->protected_->ogg_encoder_aspect);
 }
 
 FLAC__StreamEncoderWriteStatus write_callback_(const FLAC__StreamEncoder *unused, const FLAC__byte buffer[], unsigned bytes, unsigned samples, unsigned current_frame, void *client_data)
 {
 	OggFLAC__StreamEncoder *encoder = (OggFLAC__StreamEncoder*)client_data;
-	ogg_packet packet;
 	const FLAC__uint64 total_samples_estimate = FLAC__stream_encoder_get_total_samples_estimate(encoder->private_->FLAC_stream_encoder);
 
 	(void)unused;
 	FLAC__ASSERT(encoder->private_->FLAC_stream_encoder == unused);
 
-	encoder->private_->samples_written += samples;
-
-	memset(&packet, 0, sizeof(packet));
-	packet.packet = (unsigned char *)buffer;
-	packet.granulepos = encoder->private_->samples_written;
-	/* WATCHOUT:
-	 * This depends on the behavior of FLAC__StreamEncoder that 'samples'
-	 * will be 0 for metadata writes.
-	 */
-	packet.packetno = (samples == 0? -1 : (int)current_frame);
-	packet.bytes = bytes;
-
-	if(encoder->private_->is_first_packet) {
-		packet.b_o_s = 1;
-		encoder->private_->is_first_packet = false;
-	}
-
-	if(total_samples_estimate > 0 && total_samples_estimate == encoder->private_->samples_written)
-		packet.e_o_s = 1;
-
-	if(ogg_stream_packetin(&encoder->private_->ogg.stream_state, &packet) != 0)
-		return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
-
-	while(ogg_stream_pageout(&encoder->private_->ogg.stream_state, &encoder->private_->ogg.page) != 0) {
-		if(encoder->private_->write_callback(encoder, encoder->private_->ogg.page.header, encoder->private_->ogg.page.header_len, 0, current_frame, encoder->private_->client_data) != FLAC__STREAM_ENCODER_WRITE_STATUS_OK)
-			return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
-
-		if(encoder->private_->write_callback(encoder, encoder->private_->ogg.page.body, encoder->private_->ogg.page.body_len, 0, current_frame, encoder->private_->client_data) != FLAC__STREAM_ENCODER_WRITE_STATUS_OK)
-			return FLAC__STREAM_ENCODER_WRITE_STATUS_FATAL_ERROR;
-	}
-
-	return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
+	return OggFLAC__ogg_encoder_aspect_write_callback_wrapper(&encoder->protected_->ogg_encoder_aspect, total_samples_estimate, buffer, bytes, samples, current_frame, (OggFLAC__OggEncoderAspectWriteCallbackProxy)encoder->private_->write_callback, encoder, encoder->private_->client_data);
 }
 
-void metadata_callback_(const FLAC__StreamEncoder *encoder, const FLAC__StreamMetadata *metadata, void *client_data)
+void metadata_callback_(const FLAC__StreamEncoder *unused, const FLAC__StreamMetadata *metadata, void *client_data)
 {
-	/*
-	 * We don't try to go back an update metadata blocks by mucking
-	 * around inside the Ogg layer.  Maybe someday we will care to
-	 * and an OggFLAC__SeekableStreamEncoder and OggFLAC__FileEncoder
-	 * will be possible but it may just never be useful.
-	 */
-	(void)encoder, (void)metadata, (void)client_data;
+	OggFLAC__StreamEncoder *encoder = (OggFLAC__StreamEncoder*)client_data;
+
+	(void)unused;
+	FLAC__ASSERT(encoder->private_->FLAC_stream_encoder == unused);
+
+	encoder->private_->metadata_callback(encoder, metadata, encoder->private_->client_data);
 }
