@@ -1,5 +1,5 @@
 /* metaflac - Command-line FLAC metadata editor
- * Copyright (C) 2001,2002,2003  Josh Coalson
+ * Copyright (C) 2001,2002,2003,2004  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -56,7 +56,7 @@ FLAC__bool do_shorthand_operation__vorbis_comment(const char *filename, FLAC__bo
 			while(FLAC__metadata_iterator_next(iterator))
 				;
 			if(!FLAC__metadata_iterator_insert_block_after(iterator, block)) {
-				fprintf(stderr, "%s: ERROR: adding new VORBIS_COMMENT block to metadata, status =\"%s\"\n", filename, FLAC__Metadata_ChainStatusString[FLAC__metadata_chain_status(chain)]);
+				print_error_with_chain_status(chain, "%s: ERROR: adding new VORBIS_COMMENT block to metadata", filename);
 				return false;
 			}
 			/* iterator is left pointing to new block */
@@ -179,10 +179,10 @@ FLAC__bool set_vc_field(const char *filename, FLAC__StreamMetadata *block, const
 	FLAC__ASSERT(0 != needs_write);
 
 	if(raw) {
-		entry.entry = field->field;
+		entry.entry = (FLAC__byte *)field->field;
 	}
 	else if(utf8_encode(field->field, &converted) >= 0) {
-		entry.entry = converted;
+		entry.entry = (FLAC__byte *)converted;
 		needs_free = true;
 	}
 	else {
@@ -190,7 +190,7 @@ FLAC__bool set_vc_field(const char *filename, FLAC__StreamMetadata *block, const
 		return false;
 	}
 
-	entry.length = strlen(entry.entry);
+	entry.length = strlen((const char *)entry.entry);
 
 	if(!FLAC__metadata_object_vorbiscomment_insert_comment(block, block->data.vorbis_comment.num_comments, entry, /*copy=*/true)) {
 		if(needs_free)
