@@ -1,5 +1,5 @@
 /* metaflac - Command-line FLAC metadata editor
- * Copyright (C) 2001,2002,2003  Josh Coalson
+ * Copyright (C) 2001,2002,2003,2004  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -55,15 +55,23 @@ struct share__option long_options_[] = {
 	{ "set-sample-rate", 1, 0, 0 }, /* undocumented */
 	{ "set-channels", 1, 0, 0 }, /* undocumented */
 	{ "set-bps", 1, 0, 0 }, /* undocumented */
-	{ "set-total-samples", 1, 0, 0 }, /* undocumented */
-	{ "show-vc-vendor", 0, 0, 0 },
-	{ "show-vc-field", 1, 0, 0 },
-	{ "remove-vc-all", 0, 0, 0 },
-	{ "remove-vc-field", 1, 0, 0 },
-	{ "remove-vc-firstfield", 1, 0, 0 },
-	{ "set-vc-field", 1, 0, 0 },
-	{ "import-vc-from", 1, 0, 0 },
-	{ "export-vc-to", 1, 0, 0 },
+	{ "set-total-samples", 1, 0, 0 }, /* undocumented */ /* WATCHOUT: used by test/test_flac.sh on windows */
+	{ "show-vendor-tag", 0, 0, 0 }, 
+	{ "show-tag", 1, 0, 0 }, 
+	{ "remove-all-tags", 0, 0, 0 }, 
+	{ "remove-tag", 1, 0, 0 }, 
+	{ "remove-first-tag", 1, 0, 0 }, 
+	{ "set-tag", 1, 0, 0 }, 
+	{ "import-tags-from", 1, 0, 0 }, 
+	{ "export-tags-to", 1, 0, 0 }, 
+	{ "show-vc-vendor", 0, 0, 0 }, /* deprecated */
+	{ "show-vc-field", 1, 0, 0 }, /* deprecated */
+	{ "remove-vc-all", 0, 0, 0 }, /* deprecated */
+	{ "remove-vc-field", 1, 0, 0 }, /* deprecated */
+	{ "remove-vc-firstfield", 1, 0, 0 }, /* deprecated */
+	{ "set-vc-field", 1, 0, 0 }, /* deprecated */
+	{ "import-vc-from", 1, 0, 0 }, /* deprecated */
+	{ "export-vc-to", 1, 0, 0 }, /* deprecated */
 	{ "import-cuesheet-from", 1, 0, 0 },
 	{ "export-cuesheet-to", 1, 0, 0 },
 	{ "add-seekpoint", 1, 0, 0 },
@@ -441,11 +449,15 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 		else
 			undocumented_warning(opt);
 	}
-	else if(0 == strcmp(opt, "show-vc-vendor")) {
+	else if(0 == strcmp(opt, "show-vendor-tag") || 0 == strcmp(opt, "show-vc-vendor")) {
+		if(0 == strcmp(opt, "show-vc-vendor"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --show-vendor-tag\n", opt);
 		(void) append_shorthand_operation(options, OP__SHOW_VC_VENDOR);
 	}
-	else if(0 == strcmp(opt, "show-vc-field")) {
+	else if(0 == strcmp(opt, "show-tag") || 0 == strcmp(opt, "show-vc-field")) {
 		const char *violation;
+		if(0 == strcmp(opt, "show-vc-field"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --show-tag\n", opt);
 		op = append_shorthand_operation(options, OP__SHOW_VC_FIELD);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_vorbis_comment_field_name(option_argument, &(op->argument.vc_field_name.value), &violation)) {
@@ -454,11 +466,15 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 			ok = false;
 		}
 	}
-	else if(0 == strcmp(opt, "remove-vc-all")) {
+	else if(0 == strcmp(opt, "remove-all-tags") || 0 == strcmp(opt, "remove-vc-all")) {
+		if(0 == strcmp(opt, "remove-vc-all"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --remove-all-tags\n", opt);
 		(void) append_shorthand_operation(options, OP__REMOVE_VC_ALL);
 	}
-	else if(0 == strcmp(opt, "remove-vc-field")) {
+	else if(0 == strcmp(opt, "remove-tag") || 0 == strcmp(opt, "remove-vc-field")) {
 		const char *violation;
+		if(0 == strcmp(opt, "remove-vc-field"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --remove-tag\n", opt);
 		op = append_shorthand_operation(options, OP__REMOVE_VC_FIELD);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_vorbis_comment_field_name(option_argument, &(op->argument.vc_field_name.value), &violation)) {
@@ -467,8 +483,10 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 			ok = false;
 		}
 	}
-	else if(0 == strcmp(opt, "remove-vc-firstfield")) {
+	else if(0 == strcmp(opt, "remove-first-tag") || 0 == strcmp(opt, "remove-vc-firstfield")) {
 		const char *violation;
+		if(0 == strcmp(opt, "remove-vc-firstfield"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --remove-first-tag\n", opt);
 		op = append_shorthand_operation(options, OP__REMOVE_VC_FIRSTFIELD);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_vorbis_comment_field_name(option_argument, &(op->argument.vc_field_name.value), &violation)) {
@@ -477,8 +495,10 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 			ok = false;
 		}
 	}
-	else if(0 == strcmp(opt, "set-vc-field")) {
+	else if(0 == strcmp(opt, "set-tag") || 0 == strcmp(opt, "set-vc-field")) {
 		const char *violation;
+		if(0 == strcmp(opt, "set-vc-field"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --set-tag\n", opt);
 		op = append_shorthand_operation(options, OP__SET_VC_FIELD);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_vorbis_comment_field(option_argument, &(op->argument.vc_field.field), &(op->argument.vc_field.field_name), &(op->argument.vc_field.field_value), &(op->argument.vc_field.field_value_length), &violation)) {
@@ -487,7 +507,9 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 			ok = false;
 		}
 	}
-	else if(0 == strcmp(opt, "import-vc-from")) {
+	else if(0 == strcmp(opt, "import-tags-from") || 0 == strcmp(opt, "import-vc-from")) {
+		if(0 == strcmp(opt, "import-vc-from"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --import-tags-from\n", opt);
 		op = append_shorthand_operation(options, OP__IMPORT_VC_FROM);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_filename(option_argument, &(op->argument.filename.value))) {
@@ -495,7 +517,9 @@ FLAC__bool parse_option(int option_index, const char *option_argument, CommandLi
 			ok = false;
 		}
 	}
-	else if(0 == strcmp(opt, "export-vc-to")) {
+	else if(0 == strcmp(opt, "export-tags-to") || 0 == strcmp(opt, "export-vc-to")) {
+		if(0 == strcmp(opt, "export-vc-to"))
+			fprintf(stderr, "WARNING: --%s is deprecated, the new name is --export-tags-to\n", opt);
 		op = append_shorthand_operation(options, OP__EXPORT_VC_TO);
 		FLAC__ASSERT(0 != option_argument);
 		if(!parse_filename(option_argument, &(op->argument.filename.value))) {
