@@ -1,5 +1,5 @@
 /* plugin_common - Routines common to several plugins
- * Copyright (C) 2002,2003  Josh Coalson
+ * Copyright (C) 2002,2003,2004  Josh Coalson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -22,10 +22,7 @@
 #include "canonical_tag.h"
 #include "charset.h"
 #include "dither.h"
-#include "id3v1.h"
-#include "id3v2.h"
 #include "locale_hack.h"
-#include "replaygain_synthesis.h"
 #include "vorbiscomment.h"
 
 #endif
